@@ -136,7 +136,7 @@ char **dir_history;
 int dir_history_count;
 int dir_history_point;
 
-#define BUILTIN_OPERATIONS                       88
+#define BUILTIN_OPERATIONS                       89
 
 #define BUILTIN_copy				 -1
 #define BUILTIN_move				 -2
@@ -226,6 +226,7 @@ int dir_history_point;
 #define BUILTIN_horizontal_scroll_right		-86
 #define BUILTIN_select_extension		-87
 #define BUILTIN_unselect_extension		-88
+#define BUILTIN_apropos				-89
 
 #define MAX_BUILTIN_NAME			 35
 
@@ -320,6 +321,7 @@ char builtin[BUILTIN_OPERATIONS][MAX_BUILTIN_NAME] =
     "horizontal-scroll-right",
     "select-extension",
     "unselect-extension",
+    "apropos",
 };
 
 
@@ -333,8 +335,8 @@ typedef struct
     char  hide;         /* Hide the output, emulating a builtin command.  */
     char  builtin;      /* This is a builtin command.  */
     char *sequence;     /* The ascii representation of the key sequence on
-			   which the command is binded; used only for error
-			   reporting purposes.  */
+			   which the command is bound; used for error
+			   reporting and the apropos command.  */
     xstack_t *history;  /* The history of the strings used to expand the
 			   command body.  */
 } command_t;
@@ -1088,6 +1090,7 @@ il_isearch(static_text, dest, status, action)
     int *action;
 {
     int key;
+    int keycmd;
     tty_key_t *ks;
     command_t *command;
     static input_line_t *saved_il;
@@ -1130,14 +1133,14 @@ il_isearch(static_text, dest, status, action)
 	return NULL;
     }
 
-    key = ks->key_seq[0];
+    keycmd = key = ks->key_seq[0];
 
     command = (command_t *)ks->aux_data;
 
     if (command && command->builtin)
-	key = - 1 - (command->name - builtin[0]) / MAX_BUILTIN_NAME;
+	keycmd = - 1 - (command->name - builtin[0]) / MAX_BUILTIN_NAME;
 
-    switch (key)
+    switch (keycmd)
     {
 	case key_INTERRUPT:
 	case BUILTIN_action:
@@ -1164,9 +1167,9 @@ il_isearch(static_text, dest, status, action)
 	    break;
 
 	default:
-	    if ((key == BUILTIN_isearch_backward &&
+	    if ((keycmd == BUILTIN_isearch_backward &&
 		 status == IL_ISEARCH_BACKWARD)  ||
-		(key == BUILTIN_isearch_forward  &&
+		(keycmd == BUILTIN_isearch_forward  &&
 		 status == IL_ISEARCH_FORWARD))
 	    {
 		if (il_is_empty())
@@ -1185,7 +1188,7 @@ il_isearch(static_text, dest, status, action)
 	    {
 		/* Force a NULL return value when detecting non printable
 		   characters (or functional keys).  */
-		key = key_INTERRUPT;
+		keycmd = key_INTERRUPT;
 	    }
 
 	    break;
@@ -1197,7 +1200,7 @@ il_isearch(static_text, dest, status, action)
     tty_update();
     il_get_contents(dest);
 
-    return (key == BUILTIN_action || key == key_INTERRUPT) ? NULL : *dest;
+    return (keycmd == BUILTIN_action || keycmd == key_INTERRUPT) ? NULL : *dest;
 }
 
 
@@ -1811,9 +1814,9 @@ main(argc, argv)
     int output_final_path = OFF;
     input_line_t *saved_il = NULL;
     char *panel_path, *current_path;
-    char *lock_password, *unlock_password;
+    char *lock_password, *unlock_password, *aproposstr;
     int child_exit_code, repeat_count, keys;
-    int action_status, i, retval, to_case, cmp_mode, r1, r2;
+    int action_status, i, retval, to_case, cmp_mode;
     int c, ansi_colors = -1, use_last_screen_character = ON;
     int entry, key, app_end = 0, first_time = 1, errors = 0;
     char *left_panel_path, *right_panel_path, *output_string;
@@ -1917,17 +1920,26 @@ main(argc, argv)
     printf(copyright);
 
 #ifdef DEBIAN
-    add_to_environment("GIT_EDITOR",  "EDITOR",     "sensible-editor");
-    add_to_environment("GIT_PAGER",   "PAGER",      "sensible-pager");
-    add_to_environment("GIT_BROWSER", (char *)NULL, "sensible-browser");
-#else /* DEBIAN */
-    add_to_environment("GIT_EDITOR",  "EDITOR",     "vi");
-    add_to_environment("GIT_PAGER",   "PAGER",      "more");
-    add_to_environment("GIT_BROWSER", (char *)NULL, "lynx");
-#endif /* DEBIAN */
-    add_to_environment("GIT_SHELL",   "SHELL",      "/bin/sh");
-    add_to_environment("GIT_RMAIL",   (char *)NULL, "mail");
-    add_to_environment("GIT_VMSTAT",  (char *)NULL, "free");
+    add_to_environment("GIT_EDITOR",    "EDITOR",     "sensible-editor");
+    add_to_environment("GNUIT_EDITOR",  "GIT_EDITOR", "sensible-editor");
+    add_to_environment("GIT_PAGER",     "PAGER",      "sensible-pager");
+    add_to_environment("GNUIT_PAGER",   "GIT_PAGER",  "sensible-pager");
+    add_to_environment("GIT_BROWSER",   (char *)NULL, "sensible-browser");
+    add_to_environment("GNUIT_BROWSER", "GIT_BROWSER",  "sensible-browser");
+#else /* !DEBIAN */
+    add_to_environment("GIT_EDITOR",    "EDITOR",     "vi");
+    add_to_environment("GNUIT_EDITOR",  "GIT_EDITOR", "vi");
+    add_to_environment("GIT_PAGER",     "PAGER",      "more");
+    add_to_environment("GNUIT_PAGER",   "GIT_PAGER",  "more");
+    add_to_environment("GIT_BROWSER",   (char *)NULL, "lynx");
+    add_to_environment("GNUIT_BROWSER", "GIT_BROWSER",  "lynx");
+#endif /* !DEBIAN */
+    add_to_environment("GIT_SHELL",     "SHELL",      "/bin/sh");
+    add_to_environment("GNUIT_SHELL",   "GIT_SHELL",  "/bin/sh");
+    add_to_environment("GIT_RMAIL",     (char *)NULL, "mail");
+    add_to_environment("GNUIT_RMAIL",   "GIT_RMAIL",    "mail");
+    add_to_environment("GIT_VMSTAT",    (char *)NULL, "free");
+    add_to_environment("GNUIT_VMSTAT",  "GIT_VMSTAT",   "free");
 
     tty_init(TTY_RESTRICTED_INPUT);
 
@@ -1948,17 +1960,12 @@ main(argc, argv)
     if (temporary_directory == NULL)
 	temporary_directory = "/tmp";
 
-    stdout_log_name = xmalloc(32 + strlen(temporary_directory) + 1);
-    stderr_log_name = xmalloc(32 + strlen(temporary_directory) + 1);
-
-    srand(time(NULL));
-    r1 = 1 + (int) (100000000.0 * rand() / (RAND_MAX + 1.0));
-    r2 = 1 + (int) (100000000.0 * rand() / (RAND_MAX + 1.0));
-
-    sprintf(stdout_log_name, "%s/git.1.%d.%d",
-	    temporary_directory, (int)getpid(), r1);
-    sprintf(stderr_log_name, "%s/git.2.%d.%d",
-	    temporary_directory, (int)getpid(), r2);
+    stdout_log_template = xmalloc(32 + strlen(temporary_directory) + 1);
+    stderr_log_template = xmalloc(32 + strlen(temporary_directory) + 1);
+    stdout_log_name     = xmalloc(32 + strlen(temporary_directory) + 1);
+    stderr_log_name     = xmalloc(32 + strlen(temporary_directory) + 1);
+    sprintf(stdout_log_template, "%s/gnuit.1.XXXXXX", temporary_directory);
+    sprintf(stderr_log_template, "%s/gnuit.2.XXXXXX", temporary_directory);
 
     use_section("[Setup]");
 
@@ -3198,6 +3205,65 @@ main(argc, argv)
 			     NULL, 1);
 		break;
 
+	    case BUILTIN_apropos:
+		aproposstr=NULL;
+		il_read_line("Apropos: ", &aproposstr,
+			     (char *)NULL, command->history);
+
+		if (aproposstr)
+		{
+		    if(*aproposstr != '\0')
+		    {
+			extern tty_key_t *key_list_head;
+			tty_key_t *key;
+			int fd;
+			FILE *fp=NULL;
+			int gotmatch=0;
+			char *template="gnuit-apropos-XXXXXX";
+			char *tmpfn;
+			tmpfn=xmalloc(strlen(temporary_directory)+strlen(template)+1+2);
+			sprintf(tmpfn,"%s/%s",temporary_directory,template);
+			fd=mkstemp(tmpfn);
+			if(fd != -1)
+			    fp=fdopen(fd,"w");
+			if(!fp)
+			{
+			    il_read_char("Error opening temporary file", NULL, IL_ERROR|IL_BEEP);
+			    break;
+			}
+			for(key=key_list_head;key;key=key->next)
+			{
+			    command_t *command=(command_t *)key->aux_data;
+			    if(strcasestr(command->name, aproposstr))
+			    {
+				gotmatch=1;
+				fprintf(fp,"%s: %s\n",command->name, command->sequence);
+			    }
+			}
+			fclose(fp);
+			if(gotmatch)
+			{
+			    char *cmd, *pager;
+			    pager=getenv("GNUIT_PAGER");
+			    if(!pager) 
+				pager="more";
+			    cmd=xmalloc(strlen(pager)+strlen(tmpfn)+1+1);
+			    sprintf(cmd,"%s %s",pager,tmpfn);
+			    start(cmd,0);
+			    xfree(cmd);
+			    wait_msg=1;
+			}
+			else
+			    il_read_char("No matches", NULL, 0);
+			unlink(tmpfn);
+			xfree(tmpfn);
+		    }
+		    xfree(aproposstr);
+		    goto restart;
+		}
+
+		break;
+
 	    default:
 		if (key)
 		{
